@@ -20,6 +20,7 @@
 #include "statistics.h"
 #include "streambuf.h"
 #include "str.h"
+#include "homer.h"
 #include "tcp_listener.h"
 #include "main.h"
 
@@ -28,6 +29,7 @@ mutex_t tcp_connections_lock;
 GHashTable *rtpe_cngs_hash;
 GHashTable *tcp_connections_hash;
 static struct cookie_cache ng_cookie_cache;
+static bool trace_ng = false;
 
 const char magic_load_limit_strings[__LOAD_LIMIT_MAX][64] = {
 	[LOAD_LIMIT_MAX_SESSIONS] = "Parallel session limit reached",
@@ -62,6 +64,67 @@ const char *ng_command_strings_short[NGC_COUNT] = {
 	"Pub", "SubReq", "SubAns", "Unsub",
 };
 
+typedef struct ng_ctx {
+	str callid;
+	int command;
+	str cookie;
+	bool should_trace;
+	const endpoint_t *sin_ep;
+	const endpoint_t *local_ep;
+} ng_ctx;
+
+#define CH(func, ...) do { \
+	if (trace_ng) \
+		func( __VA_ARGS__); \
+} while (0)
+
+void init_ng_tracing(void) {
+	if (rtpe_config.homer_ng_on &&  has_homer())
+		trace_ng = true;
+}
+
+static GString *create_homer_msg(str *cookie, str *data) {
+	GString *msg = g_string_sized_new(cookie->len + 1 + data->len);
+	g_string_append_printf(msg, "%.*s %.*s", STR_FMT(cookie), STR_FMT(data));
+	return msg;
+}
+
+static bool should_trace_msg(enum ng_command command) {
+	switch (command) {
+		case NGC_PING:
+			return false;
+		default:
+			return true;
+	}
+}
+
+static void homer_fill_values(ng_ctx *hctx, str *callid, int command) {
+	if (hctx) {
+		hctx->command = command;
+		hctx->callid = *callid;
+	}
+}
+
+static void homer_trace_msg_in(ng_ctx *hctx, str *data) {
+	if (hctx) {
+		hctx->should_trace = should_trace_msg(hctx->command);
+		if (hctx->should_trace)	{
+			struct timeval tv;
+			gettimeofday(&tv, NULL);
+			GString *msg = create_homer_msg(&hctx->cookie, data);
+			homer_send(msg, &hctx->callid, hctx->sin_ep, hctx->local_ep, &tv, rtpe_config.homer_ng_capt_proto);
+		}
+	}
+}
+
+static void homer_trace_msg_out(ng_ctx *hctx, str *data) {
+	if (hctx && hctx->should_trace) {
+		struct timeval tv;
+		gettimeofday(&tv, NULL);
+		GString *msg = create_homer_msg(&hctx->cookie, data);
+		homer_send(msg, &hctx->callid, hctx->local_ep, hctx->sin_ep, &tv, rtpe_config.homer_ng_capt_proto);
+	}
+}
 
 static void pretty_print(bencode_item_t *el, GString *s) {
 	bencode_item_t *chld;
@@ -130,14 +193,14 @@ struct control_ng_stats* get_control_ng_stats(const sockaddr_t *addr) {
 }
 
 static void __ng_buffer_free(void *p) {
-	struct ng_buffer *ngbuf = p;
+	ng_buffer *ngbuf = p;
 	bencode_buffer_free(&ngbuf->buffer);
 	if (ngbuf->ref)
 		obj_put_o(ngbuf->ref);
 }
 
-struct ng_buffer *ng_buffer_new(struct obj *ref) {
-	struct ng_buffer *ngbuf = obj_alloc0("ng_buffer", sizeof(*ngbuf), __ng_buffer_free);
+ng_buffer *ng_buffer_new(struct obj *ref) {
+	ng_buffer *ngbuf = obj_alloc0("ng_buffer", sizeof(*ngbuf), __ng_buffer_free);
 	if (ref)
 		ngbuf->ref = obj_get_o(ref); // hold until we're done
 
@@ -148,68 +211,50 @@ struct ng_buffer *ng_buffer_new(struct obj *ref) {
 	return ngbuf;
 }
 
-int control_ng_process(str *buf, const endpoint_t *sin, char *addr, const sockaddr_t *local,
-		void (*cb)(str *, str *, const endpoint_t *, const sockaddr_t *, void *),
-		void *p1, struct obj *ref)
+static void control_ng_process_payload(ng_ctx *hctx, str *reply, str *data, const endpoint_t *sin, char *addr, struct obj *ref,
+		struct ng_buffer **ngbufp)
 {
-	AUTO_CLEANUP(struct ng_buffer *ngbuf, ng_buffer_auto_release) = NULL;
 	bencode_item_t *dict, *resp;
-	str cmd = STR_NULL, cookie, data, reply, *to_send, callid;
+	str cmd = STR_NULL, callid;
 	const char *errstr, *resultstr;
 	GString *log_str;
 	struct timeval cmd_start, cmd_stop, cmd_process_time;
 	struct control_ng_stats* cur = get_control_ng_stats(&sin->address);
-	int funcret = -1;
 	enum ng_command command = -1;
 
-	str_chr_str(&data, buf, ' ');
-	if (!data.s || data.s == buf->s) {
-		ilogs(control, LOG_WARNING, "Received invalid data on NG port (no cookie) from %s: " STR_FORMAT_M,
-				addr, STR_FMT_M(buf));
-		return funcret;
-	}
-
-	ngbuf = ng_buffer_new(ref);
+	struct ng_buffer *ngbuf = *ngbufp = ng_buffer_new(ref);
 
 	resp = bencode_dictionary(&ngbuf->buffer);
 	assert(resp != NULL);
 
 	str *(*collapse_func)(bencode_item_t *root, str *out) = bencode_collapse_str;
 
-	cookie = *buf;
-	cookie.len -= data.len;
-	*data.s++ = '\0';
-	data.len--;
-
 	errstr = "Invalid data (no payload)";
-	if (data.len <= 0)
+	if (data->len <= 0)
 		goto err_send;
 
-	to_send = cookie_cache_lookup(&ng_cookie_cache, &cookie);
-	if (to_send) {
-		ilogs(control, LOG_INFO, "Detected command from %s as a duplicate", addr);
-		resp = NULL;
-		goto send_only;
-	}
-
-	if (data.s[0] == 'd') {
-		dict = bencode_decode_expect_str(&ngbuf->buffer, &data, BENCODE_DICTIONARY);
+	/* Bencode dictionary */
+	if (data->s[0] == 'd') {
+		dict = bencode_decode_expect_str(&ngbuf->buffer, data, BENCODE_DICTIONARY);
 		errstr = "Could not decode bencode dictionary";
 		if (!dict)
 			goto err_send;
 	}
-	else if (data.s[0] == '{') {
+
+	/* JSON */
+	else if (data->s[0] == '{') {
 		collapse_func = bencode_collapse_str_json;
 		JsonParser *json = json_parser_new();
 		bencode_buffer_destroy_add(&ngbuf->buffer, g_object_unref, json);
 		errstr = "Failed to parse JSON document";
-		if (!json_parser_load_from_data(json, data.s, data.len, NULL))
+		if (!json_parser_load_from_data(json, data->s, data->len, NULL))
 			goto err_send;
 		dict = bencode_convert_json(&ngbuf->buffer, json);
 		errstr = "Could not decode bencode dictionary";
 		if (!dict || dict->type != BENCODE_DICTIONARY)
 			goto err_send;
 	}
+
 	else {
 		errstr = "Invalid NG data format";
 		goto err_send;
@@ -346,6 +391,9 @@ int control_ng_process(str *buf, const endpoint_t *sin, char *addr, const sockad
 			errstr = "Unrecognized command";
 	}
 
+	CH(homer_fill_values, hctx, &callid, command);
+	CH(homer_trace_msg_in, hctx, data);
+
 	// stop command timer
 	gettimeofday(&cmd_stop, NULL);
 	//print command duration
@@ -373,7 +421,7 @@ err_send:
 
 	if (errstr < magic_load_limit_strings[0] || errstr > magic_load_limit_strings[__LOAD_LIMIT_MAX-1]) {
 		ilogs(control, LOG_WARNING, "Protocol error in packet from %s: %s [" STR_FORMAT_M "]",
-				addr, errstr, STR_FMT_M(&data));
+				addr, errstr, STR_FMT_M(data));
 		bencode_dictionary_add_string(resp, "result", "error");
 		bencode_dictionary_add_string(resp, "error-reason", errstr);
 		g_atomic_int_inc(&cur->errors);
@@ -385,14 +433,13 @@ err_send:
 	}
 
 send_resp:
-	collapse_func(resp, &reply);
-	to_send = &reply;
+	collapse_func(resp, reply);
 
 	if (cmd.s) {
 		ilogs(control, LOG_INFO, "Replying to '"STR_FORMAT"' from %s (elapsed time %llu.%06llu sec)", STR_FMT(&cmd), addr, (unsigned long long)cmd_process_time.tv_sec, (unsigned long long)cmd_process_time.tv_usec);
 
 		if (get_log_level(control) >= LOG_DEBUG) {
-			dict = bencode_decode_expect_str(&ngbuf->buffer, to_send, BENCODE_DICTIONARY);
+			dict = bencode_decode_expect_str(&ngbuf->buffer, reply, BENCODE_DICTIONARY);
 			if (dict) {
 				log_str = g_string_sized_new(256);
 				g_string_append_printf(log_str, "Response dump for '"STR_FORMAT"' to %s: %s",
@@ -406,21 +453,76 @@ send_resp:
 		}
 	}
 
-send_only:
-	funcret = 0;
-	cb(&cookie, to_send, sin, local, p1);
-
-	if (resp)
-		cookie_cache_insert(&ng_cookie_cache, &cookie, &reply);
-	else
-		free(to_send);
-
-	goto out;
-
-out:
 	release_closed_sockets();
 	log_info_pop_until(&callid);
-	return funcret;
+	CH(homer_trace_msg_out ,hctx, reply);
+}
+
+int control_ng_process(str *buf, const endpoint_t *sin, char *addr, const sockaddr_t *local,
+		void (*cb)(str *, str *, const endpoint_t *, const sockaddr_t *, void *),
+		void *p1, struct obj *ref)
+{
+	str data;
+	str_chr_str(&data, buf, ' ');
+	if (!data.s || data.s == buf->s) {
+		ilogs(control, LOG_WARNING, "Received invalid NG data (no cookie) from %s: " STR_FORMAT_M,
+				addr, STR_FMT_M(buf));
+		return -1;
+	}
+
+	str cookie = *buf;
+	cookie.len -= data.len;
+	*data.s++ = '\0';
+	data.len--;
+
+	cache_entry *cached = cookie_cache_lookup(&ng_cookie_cache, &cookie);
+	if (cached) {
+		ilogs(control, LOG_INFO, "Detected command from %s as a duplicate", addr);
+
+		ng_ctx hctx  = {.sin_ep = sin,
+				.local_ep = p1 ? &(((socket_t*)p1)->local) : NULL,
+				.cookie = cookie,
+				.command = cached->command,
+				.callid = *cached->callid,
+				.should_trace = should_trace_msg(cached->command)};
+
+		CH(homer_trace_msg_in, &hctx, &data);
+		cb(&cookie, cached->reply, sin, local, p1);
+		CH(homer_trace_msg_out, &hctx, cached->reply);
+
+		cache_entry_free(cached);
+		return 0;
+	}
+
+	str reply;
+	g_autoptr(ng_buffer) ngbuf = NULL;
+
+	ng_ctx hctx = {.sin_ep = sin,
+			.local_ep = p1 ? &(((socket_t*)p1)->local) : NULL,
+			.cookie = cookie,
+			.command = -1};
+
+	control_ng_process_payload(trace_ng ? &hctx : NULL,
+								&reply, &data, sin, addr, ref, &ngbuf);
+
+	cb(&cookie, &reply, sin, local, p1);
+	cache_entry ce = {.reply = &reply, .command = hctx.command, .callid = &hctx.callid};
+	cookie_cache_insert(&ng_cookie_cache, &cookie, &ce);
+
+	return 0;
+}
+
+int control_ng_process_plain(str *data, const endpoint_t *sin, char *addr, const sockaddr_t *local,
+		void (*cb)(str *, str *, const endpoint_t *, const sockaddr_t *, void *),
+		void *p1, struct obj *ref)
+{
+	g_autoptr(ng_buffer) ngbuf = NULL;
+
+	str reply;
+	control_ng_process_payload(NULL, &reply, data, sin, addr, ref, &ngbuf);
+	cb(NULL, &reply, sin, local, p1);
+
+	return 0;
 }
 
 INLINE void control_ng_send_generic(str *cookie, str *body, const endpoint_t *sin, const sockaddr_t *from,
@@ -545,8 +647,8 @@ void control_ng_free(void *p) {
 		g_hash_table_destroy(rtpe_cngs_hash);
 		rtpe_cngs_hash = NULL;
 	}
-	poller_del_item(rtpe_poller, c->udp_listener.fd);
-	close_socket(&c->udp_listener);
+	rtpe_poller_del_item(rtpe_control_poller, c->udp_listener.fd);
+	reset_socket(&c->udp_listener);
 	streambuf_listener_shutdown(&c->tcp_listener);
 	if (tcp_connections_hash)
 		g_hash_table_destroy(tcp_connections_hash);
@@ -610,11 +712,11 @@ void notify_ng_tcp_clients(str *data) {
 	mutex_unlock(&tcp_connections_lock);
 }
 
-void control_ng_init() {
+void control_ng_init(void) {
 	mutex_init(&rtpe_cngs_lock);
 	rtpe_cngs_hash = g_hash_table_new(sockaddr_t_hash, sockaddr_t_eq);
 	cookie_cache_init(&ng_cookie_cache);
 }
-void control_ng_cleanup() {
+void control_ng_cleanup(void) {
 	cookie_cache_cleanup(&ng_cookie_cache);
 }
