@@ -250,12 +250,13 @@ struct send_timer *send_timer_new(struct packet_stream *ps) {
 
 // call is locked in R
 // ssrc_out is locked
+// st->sink is locked
 static void send_timer_rtcp(struct send_timer *st, struct ssrc_ctx *ssrc_out) {
 	struct call_media *media = st->sink ? st->sink->media : NULL;
 	if (!media)
 		return;
 
-	rtcp_send_report(media, ssrc_out);
+	rtcp_send_report(media, ssrc_out, st->sink);
 
 	ssrc_out->next_rtcp = rtpe_now;
 	timeval_add_usec(&ssrc_out->next_rtcp, 5000000 + (ssl_random() % 2000000));
@@ -351,6 +352,7 @@ out:
 	log_info_pop();
 }
 
+// sink->lock is held
 static void __send_timer_rtcp(struct send_timer *st, struct ssrc_ctx *ssrc_out) {
 	// do we send RTCP?
 	if (!ssrc_out)
@@ -375,13 +377,14 @@ static void send_timer_send_lock(struct send_timer *st, struct codec_packet *cp)
 
 	log_info_call(call);
 	rwlock_lock_r(&call->master_lock);
-	mutex_lock(&st->sink->out_lock);
+	mutex_lock(&st->sink->lock);
 
 	__send_timer_send_common(st, cp);
 
-	mutex_unlock(&st->sink->out_lock);
-
 	__send_timer_rtcp(st, ssrc_out);
+
+	mutex_unlock(&st->sink->lock);
+
 	ssrc_ctx_put(&ssrc_out);
 
 	rwlock_unlock_r(&call->master_lock);
@@ -396,8 +399,15 @@ static void send_timer_send_nolock(struct send_timer *st, struct codec_packet *c
 
 	log_info_call(call);
 
+	struct ssrc_ctx *ssrc_out = cp->ssrc_out;
+	if (ssrc_out)
+		ssrc_ctx_hold(ssrc_out);
+
 	__send_timer_send_common(st, cp);
 
+	__send_timer_rtcp(st, ssrc_out);
+	ssrc_ctx_put(&ssrc_out);
+
 	log_info_pop();
 }
 
@@ -522,10 +532,10 @@ retry:;
 
 	media_packet_encrypt(mp->crypt_handler->out->rtp_crypt, mp->sink, &packet);
 
-	mutex_lock(&mp->sink->out_lock);
+	mutex_lock(&mp->sink->lock);
 	if (media_socket_dequeue(&packet, mp->sink))
 		ilog(LOG_ERR, "Error sending playback media to RTP sink");
-	mutex_unlock(&mp->sink->out_lock);
+	mutex_unlock(&mp->sink->lock);
 
 	// schedule our next run
 	timeval_add_usec(&mp->next_run, us_dur);
@@ -987,10 +997,10 @@ void media_player_add_packet(struct media_player *mp, char *buf, size_t len,
 
 	media_packet_encrypt(mp->crypt_handler->out->rtp_crypt, mp->sink, &packet);
 
-	mutex_lock(&mp->sink->out_lock);
+	mutex_lock(&mp->sink->lock);
 	if (media_socket_dequeue(&packet, mp->sink))
 		ilog(LOG_ERR, "Error sending playback media to RTP sink");
-	mutex_unlock(&mp->sink->out_lock);
+	mutex_unlock(&mp->sink->lock);
 
 	timeval_add_usec(&mp->next_run, us_dur);
 	timerthread_obj_schedule_abs(&mp->tt_obj, &mp->next_run);
