@@ -10,8 +10,6 @@ mix_t *mix_new(void);
 void mix_destroy(mix_t *mix);
 int mix_config(mix_t *, const format_t *format);
 int mix_add(mix_t *mix, AVFrame *frame, unsigned int idx, void *, output_t *output);
-unsigned int mix_get_index(mix_t *, void *);
-
-
+unsigned int mix_get_index(mix_t *, void *, unsigned int);
 #endif
 
