@@ -64,12 +64,12 @@ static void do_notify(void *p, void *u) {
 
 	/* allow redirects */
 	err = "setting CURLOPT_FOLLOWLOCATION";
-	if ((ret = curl_easy_setopt(c, CURLOPT_FOLLOWLOCATION, 1)) != CURLE_OK)
+	if ((ret = curl_easy_setopt(c, CURLOPT_FOLLOWLOCATION, 1L)) != CURLE_OK)
 		goto fail;
 
 	/* max 5 redirects */
 	err = "setting CURLOPT_MAXREDIRS";
-	if ((ret = curl_easy_setopt(c, CURLOPT_MAXREDIRS, 5)) != CURLE_OK)
+	if ((ret = curl_easy_setopt(c, CURLOPT_MAXREDIRS, 5L)) != CURLE_OK)
 		goto fail;
 
 	/* add headers */
@@ -80,14 +80,14 @@ static void do_notify(void *p, void *u) {
 	/* POST vs GET */
 	if (notify_post) {
 		err = "setting CURLOPT_POST";
-		if ((ret = curl_easy_setopt(c, CURLOPT_POST, 1)) != CURLE_OK)
+		if ((ret = curl_easy_setopt(c, CURLOPT_POST, 1L)) != CURLE_OK)
 			goto fail;
 	}
 
 	/* cert verify (enabled by default) */
 	if (notify_nverify) {
 		err = "setting CURLOPT_SSL_VERIFYPEER";
-		if ((ret = curl_easy_setopt(c, CURLOPT_SSL_VERIFYPEER, 0)) != CURLE_OK)
+		if ((ret = curl_easy_setopt(c, CURLOPT_SSL_VERIFYPEER, 0L)) != CURLE_OK)
 			goto fail;
 	}
 
@@ -212,7 +212,7 @@ static void *notify_timer(void *p) {
 		gettimeofday(&now, NULL);
 		if (now.tv_sec < first->retry_time) {
 			ilog(LOG_DEBUG, "Sleeping until next scheduled HTTP notification retry in %lu seconds",
-					(unsigned long) first->retry_time - now.tv_sec);
+					(unsigned long) (first->retry_time - now.tv_sec));
 			struct timespec ts = {.tv_sec = first->retry_time, .tv_nsec = 0};
 			pthread_cond_timedwait(&timer_cond, &timer_lock, &ts);
 			continue;
